var sys = require('sys');
var ndns = require('../lib/ndns');
var redis = require('../lib/redis');

var server = ndns.createServer('udp4');

var debug;
var debugLevel = parseInt(process.env.NODE_DEBUG, 16);
if(debugLevel & 0x4) {
    debug = function (x) { sys.error('redis: ' + x); };
} else {
    debug = function () { };
}

var outgoing = [];

var type = null, mbulklen = null, bulklen = null;
var data = "";

var db = redis.createClient(6379, "127.0.0.1");
db.onCommandType = function (commandType) {
    debug('db.onCommandType');
    type = commandType;
};
db.onMBulkLength = function (len) {
    debug('db.onMBulkLength');
    mbulklen = len;
};
db.onBulkLength = function (len) {
    debug('db.onBulkLength');
    bulklen = len;
};
db.onData = function (b, start, end) {
    debug('db.onData');
    data += b.toString("ascii", start, end);
};
db.onDataEnd = function () {
    debug('db.onDataEnd');
    debug('data: ' + data);
    data = "";
    bulklen = null;
};
db.onMBulkEnd = function () {
    debug('db.onMBulkEnd');
    var res = outgoing.shift();
    if (res.header.ancount == 0) {
	res.header.rcode = ndns.ns_rcode.nxdomain;
    }
    res.send();
    mbulklen = null;
};

function Options () {
};

Options.prototype.port = 5300;
Options.prototype.listen_on = [ "127.0.0.1" ];
Options.prototype.listen_on_v6 [ "::1" ];
Options.prototype.recursion = "no";

var ns_options = new Options();

function setOptions (options) {
    for (var k in Options.prototype) {
	if (options[k]) {
	    ns_options[k] = options[k];
	}
    }
};
exports.setOptions = setOptions;

server.on("request", function(req, res) {
    var key = "rr:";

    for (var i = 0; i < req.q.length; i++) {
	res.q.add(req.q[i]);
	key += (req.q[i].type.toString() + ":" +
		req.q[i].class.toString() + ":" +
		req.q[i].name);
    }

    res.setHeader(req.header);
    res.header.qr = 1;
    res.header.aa = 1;
    res.header.rd = 0;
    res.header.ra = 0;

    if (key.length) {
	outgoing.push(res);
	db.query("lrange", key, 0, -1);
    } else {
	// nxdomain
	res.header.rcode = ndns.ns_rcode.nxdomain;
	res.send();
    }
});
server.bind(5353);
